@@ -9,13 +9,7 @@ define('VERIFICATION_REQUIRED', false);
 // Maximum allowed size in KB of source submissions.
 define('SOURCESIZE', 256);
 
-// Penalty time in minutes per wrong submission (if finally solved).
-define('PENALTY_TIME', 20);
-
-// Internal and output character set used, don't change.
-define('DJ_CHARACTER_SET', 'utf-8');
-
-// Possible exitcodes from test_solution.sh and their meaning
+// Possible exitcodes from testcase_run.sh and their meaning.
 $EXITCODES = array (
 	0   => 'correct',
 	101 => 'compiler-error',
@@ -26,7 +20,12 @@ $EXITCODES = array (
 	106 => 'presentation-error',
 	107 => 'memory-limit',
 	108 => 'output-limit',
-	127 => 'internal-error'
+/* Uncomment the next line to accept internal errors in the judging
+ * backend as outcome. WARNING: it is highly discouraged to enable
+ * this, as the judgehost may be in an inconsistent state after an
+ * internal error occurred and it is recommended to inspect manually.
+ */
+//	127 => 'internal-error'
 	);
 
 /** Loglevels and debugging */
