@@ -3,7 +3,7 @@
  * Common page header.
  * Before including this, one can set $title, $refresh and $popup.
  *
- * $Id: header.php 2843 2009-07-14 22:12:38Z kink $
+ * $Id: header.php 3129 2010-02-23 20:44:05Z kink $
  *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
@@ -20,6 +20,9 @@ echo '<?xml version="1.0" encoding="' . DJ_CHARACTER_SET . '" ?>' . "\n";
 if(!isset($menu)) {
 	$menu = true;
 }
+if(!isset($ajaxtitle)) {
+	$ajaxtitle = '';
+}
 ?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
 	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
@@ -27,25 +30,29 @@ if(!isset($menu)) {
 <head>
 	<!-- DOMjudge version <?php echo DOMJUDGE_VERSION?> -->
 <title><?php echo $title?></title>
-<link rel="shortcut icon" href="<?php echo getBaseURI()?>images/favicon.png" type="image/png" />
-<link rel="stylesheet" href="<?php echo getBaseURI()?>style.css" type="text/css" />
+<link rel="shortcut icon" href="../images/favicon.png" type="image/png" />
+<link rel="stylesheet" href="../style.css" type="text/css" />
 <?php
 if ( IS_JURY ) {
 	echo "<link rel=\"stylesheet\" href=\"style_jury.css\" type=\"text/css\" />\n";
 	if (isset($printercss)) {
 		echo "<link rel=\"stylesheet\" href=\"style_printer.css\" type=\"text/css\" media=\"print\" />\n";
 	}
+	if (isset($jscolor)) {
+		echo "<script type=\"text/javascript\" src=\"" .
+		"../js/jscolor.js\"></script>\n";
+	}
 	echo "<script type=\"text/javascript\" src=\"" .
-		getBaseURI() . "sorttable.js\"></script>\n";
+		"../js/sorttable.js\"></script>\n";
 }
 
 /* NOTE: here a local menu.php is included
  *       both jury and team have their own menu.php
  */
 if ($menu) {?>
-<script type="text/javascript" src="<?php echo getBaseURI()?>ajax.js"></script>
+<script type="text/javascript" src="../js/domjudge.js"></script>
 </head>
-<body onload="setInterval('updateClarifications()', 20000)">
+<body onload="setInterval('updateClarifications(\'<?php echo $ajaxtitle?>\')', 20000)">
 <?php include("menu.php");
 } else {?>
 </head>
