@@ -3,19 +3,44 @@
 /**
  * Functions for calculating the scoreboard.
  *
- * $Id: scoreboard.php 3041 2009-12-29 16:23:05Z kink $
+ * $Id: scoreboard.php 3209 2010-06-12 00:13:43Z eldering $
  *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
 
 
-/** 
+/**
  * The calcScoreRow is in lib/lib.misc.php because it's used by other
  * parts of the system aswell.
  */
 require_once(LIBDIR . '/lib.misc.php');
 
+/**
+ * Calculate the penalty time.
+ *
+ * This expects bool $solved (whether there was at least one correct
+ * submission by this team for this problem) and int $num_submissions
+ * (the total number of tries for this problem by this team)
+ * as input, uses the constant PENALTY_TIME and outputs the number
+ * of penalty minutes.
+ *
+ * The current formula is as follows:
+ * - Penalty time is only counted for problems that the team finally
+ *   solved. Yet unsolved problems always have zero penalty minutes.
+ * - The penalty is PENALTY_TIME (usually 20 minutes) for each
+ *   unsuccessful try. By definition, the number of unsuccessful
+ *   tries is the number of submissions for a problem minus 1: the
+ *   final, correct one.
+ */
+
+function calcPenaltyTime($solved, $num_submissions)
+{
+	if ( ! $solved ) return 0;
+
+	return ( $num_submissions - 1 ) * PENALTY_TIME;
+}
+
 /**
  * Generate scoreboard data based on the cached data in table
  * 'scoreboard_{public,jury}'. If the function is called while
@@ -52,7 +77,7 @@ function genScoreBoard($cdata) {
 
 	// Don't leak information before start of contest
 	if ( ! $cstarted && ! IS_JURY ) return;
-	
+
 	// get the teams and problems
 	$teams = $DB->q('KEYTABLE SELECT login AS ARRAYKEY, login, team.name,
 	                 team.categoryid, team.affilid, country, sortorder
@@ -62,18 +87,18 @@ function genScoreBoard($cdata) {
 	                 LEFT JOIN team_affiliation
 	                        ON (team_affiliation.affilid = team.affilid)' .
 	                ( IS_JURY ? '' : ' WHERE visible = 1' ) );
-	
+
 	$probs = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY, probid FROM problem
 	                 WHERE cid = %i AND allow_submit = 1
 	                 ORDER BY probid', $cid);
-	
+
 	// initialize the arrays we'll build from the data
 	$MATRIX = $SCORES = array();
-	$SUMMARY = array('num_correct' => 0, 
+	$SUMMARY = array('num_correct' => 0,
 	                 'affils' => array(), 'countries' => array(),
 					 'problems' => array());
 
-	// scoreboard_jury is always up to date, scoreboard_public might be frozen.	
+	// scoreboard_jury is always up to date, scoreboard_public might be frozen.
 	if ( IS_JURY || $showfinal ) {
 		$cachetable = 'scoreboard_jury';
 	} else {
@@ -97,26 +122,28 @@ function genScoreBoard($cdata) {
 		$SCORES[$login]['affilid']     = $team['affilid'];
 		$SCORES[$login]['country']     = $team['country'];
 	}
-	
+
 	// loop all info the scoreboard cache and put it in our own datastructure
 	while ( $srow = $scoredata->next() ) {
-		
+
 		// skip this row if the team or problem is not known by us
 		if ( ! array_key_exists ( $srow['teamid'], $teams ) ||
 		     ! array_key_exists ( $srow['probid'], $probs ) ) continue;
-	
+
+		$penalty = calcPenaltyTime( $srow['is_correct'], $srow['submissions'] );
+
 		// fill our matrix with the scores from the database
 		$MATRIX[$srow['teamid']][$srow['probid']] = array (
 			'is_correct'      => (bool) $srow['is_correct'],
 			'num_submissions' => $srow['submissions'],
 			'time'            => $srow['totaltime'],
-			'penalty'         => $srow['penalty'] );
+			'penalty'         => $penalty );
 
 		// calculate totals for this team
 		if ( $srow['is_correct'] ) {
 			$SCORES[$srow['teamid']]['num_correct']++;
 			$SCORES[$srow['teamid']]['solve_times'][] = $srow['totaltime'];
-			$SCORES[$srow['teamid']]['total_time'] += $srow['totaltime'] + $srow['penalty'];
+			$SCORES[$srow['teamid']]['total_time'] += $srow['totaltime'] + $penalty;
 		}
 	}
 
@@ -146,7 +173,7 @@ function genScoreBoard($cdata) {
 		$SUMMARY['num_correct'] += $totals['num_correct'];
 		if ( ! empty($teams[$team]['affilid']) ) @$SUMMARY['affils'][$totals['affilid']]++;
 		if ( ! empty($teams[$team]['country']) ) @$SUMMARY['countries'][$totals['country']]++;
-		
+
 		// for each problem
 		foreach ( array_keys($probs) as $prob ) {
 
@@ -177,10 +204,34 @@ function genScoreBoard($cdata) {
 		} else {
 			$SUMMARY['problems'][$prob]['best_time'] = NULL;
 		}
-		
+
 	}
 
-	return array( 'matrix' => $MATRIX, 'scores' => $SCORES, 'summary' => $SUMMARY );
+	// get the teams and problems
+	$teams = $DB->q('KEYTABLE SELECT login AS ARRAYKEY,
+	                 login, team.name, team.categoryid, team.affilid, sortorder,
+	                 color, country, team_affiliation.name AS affilname
+	                 FROM team
+	                 LEFT JOIN team_category
+	                        ON (team_category.categoryid = team.categoryid)
+	                 LEFT JOIN team_affiliation
+	                        ON (team_affiliation.affilid = team.affilid)' .
+	                ( IS_JURY ? '' : ' WHERE visible = 1' ) );
+	$probs = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY,
+	                 probid, name, color FROM problem
+	                 WHERE cid = %i AND allow_submit = 1
+	                 ORDER BY probid', $cid);
+	$categs = $DB->q('KEYTABLE SELECT categoryid AS ARRAYKEY,
+ 	                  categoryid, name, color FROM team_category ' .
+	                 (IS_JURY ? '' : 'WHERE visible = 1 ' ) .
+	                 'ORDER BY sortorder,name,categoryid');
+
+	return array( 'matrix'     => $MATRIX,
+	              'scores'     => $SCORES,
+	              'summary'    => $SUMMARY,
+	              'teams'      => $teams,
+	              'problems'   => $probs,
+	              'categories' => $categs );
 }
 
 /**
@@ -190,25 +241,32 @@ function genScoreBoard($cdata) {
  * will always be current, regardless of the freezetime setting in the
  * contesttable. $static generates output suitable for static html pages.
  */
-function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
+function renderScoreBoard($cdata, $sdata, $myteamid = null, $static = FALSE) {
 
-	global $DB;
-
-	if ( empty( $cdata ) ) { echo "<p><em>No active contest</em></p>\n"; return; }
+	if ( empty( $cdata ) ) { echo "<p class=\"nodata\">No active contest</p>\n"; return; }
 	$cid = $cdata['cid'];
 
+	// 'unpack' the scoreboard data:
+	$scores  = $sdata['scores'];
+	$matrix  = $sdata['matrix'];
+	$summary = $sdata['summary'];
+	$teams   = $sdata['teams'];
+	$probs   = $sdata['problems'];
+	$categs  = $sdata['categories'];
+	unset($sdata);
+
 	// Show final scores if contest is over and unfreezetime has been
 	// reached, or if contest is over and no freezetime had been set.
 	// We can compare $now and the dbfields stringwise.
 	$now = now();
 	$showfinal  = ( !isset($cdata['freezetime']) &&
-		difftime($cdata['endtime'],$now) <= 0 ) ||
-		( isset($cdata['unfreezetime']) &&
-		difftime($cdata['unfreezetime'], $now) <= 0 );
+	                difftime($cdata['endtime'],$now) <= 0 ) ||
+	              ( isset($cdata['unfreezetime']) &&
+	                difftime($cdata['unfreezetime'], $now) <= 0 );
 	// freeze scoreboard if freeze time has been reached and
 	// we're not showing the final score yet
 	$showfrozen = !$showfinal && isset($cdata['freezetime']) &&
-		difftime($cdata['freezetime'],$now) <= 0;
+	              difftime($cdata['freezetime'],$now) <= 0;
 	// contest is active but has not yet started
 	$cstarted = difftime($cdata['starttime'],$now) <= 0;
 
@@ -235,24 +293,9 @@ function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
 		}
 		echo "</h4>\n\n";
 	}
-	
+
 	// configuration
 	$SHOW_AFFILIATIONS = dbconfig_get('show_affiliations', 1);
-	
-        // get the teams and problems
-	$teams = $DB->q('KEYTABLE SELECT login AS ARRAYKEY,
-	                 login, team.name, team.categoryid, team.affilid, sortorder,
-	                 color, country, team_affiliation.name AS affilname
-	                 FROM team
-	                 LEFT JOIN team_category
-	                        ON (team_category.categoryid = team.categoryid)
-	                 LEFT JOIN team_affiliation
-	                        ON (team_affiliation.affilid = team.affilid)' .
-			( IS_JURY ? '' : ' WHERE visible = 1' ) );
-	$probs = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY,
-	                 probid, name, color FROM problem
-	                 WHERE cid = %i AND allow_submit = 1
-	                 ORDER BY probid', $cid);
 
 	echo '<table class="scoreboard' . (IS_JURY ? ' scoreboard_jury' : '') . "\">\n";
 
@@ -275,26 +318,18 @@ function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
 	foreach( $probs as $pr ) {
 		echo '<th title="problem \'' . htmlspecialchars($pr['name']) . '\'" scope="col">' .
 			jurylink('problem.php?id=' . urlencode($pr['probid']),
-				htmlspecialchars($pr['probid']) . 
-				(!empty($pr['color']) ? ' <span style="color: ' .
-				htmlspecialchars($pr['color']) . ';">' . BALLOON_SYM . '</span>' : '' ) 
+				htmlspecialchars($pr['probid']) .
+				(!empty($pr['color']) ? ' <img style="background-color: ' .
+				htmlspecialchars($pr['color']) . ';" alt="problem colour ' .
+				htmlspecialchars($pr['color']) . '" src="../images/circle.png" />' : '' )
 			) .
 			'</th>';
 	}
 	echo "</tr>\n</thead>\n\n<tbody>\n";
-	
-	// now get the real scoreboard data
-	$tmp = genScoreBoard($cdata);
-	if ( !empty($tmp) ) {
-		$SCORES  = $tmp['scores'];
-		$MATRIX  = $tmp['matrix'];
-		$SUMMARY = $tmp['summary'];
-	}
-	unset($tmp);
 
 	// print the main scoreboard rows
 	$prevsortorder = -1;
-	foreach( $SCORES as $team => $totals ) {
+	foreach( $scores as $team => $totals ) {
 
 		// rank, team name, total correct, total time
 		echo '<tr';
@@ -312,7 +347,7 @@ function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
 		}
 		echo '><td class="scorepl">';
 		// Only print rank when score is different from the previous team
-		if ( !isset($prevteam) || $SCORES[$prevteam]['rank']!=$totals['rank'] ) {
+		if ( !isset($prevteam) || $scores[$prevteam]['rank']!=$totals['rank'] ) {
 			echo jurylink(null,$totals['rank']);
 		} else {
 			echo jurylink(null,'');
@@ -368,19 +403,19 @@ function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
 
 			echo '<td class=';
 			// CSS class for correct/incorrect/neutral results
-			if( $MATRIX[$team][$prob]['is_correct'] ) { 
+			if( $matrix[$team][$prob]['is_correct'] ) {
 				echo '"score_correct"';
-			} elseif ( $MATRIX[$team][$prob]['num_submissions'] > 0 ) {
+			} elseif ( $matrix[$team][$prob]['num_submissions'] > 0 ) {
 				echo '"score_incorrect"';
 			} else {
 				echo '"score_neutral"';
 			}
 			// number of submissions for this problem
-			$str = $MATRIX[$team][$prob]['num_submissions'];
+			$str = $matrix[$team][$prob]['num_submissions'];
 			// if correct, print time scored
-			if( $MATRIX[$team][$prob]['is_correct'] ) {
-				$str .= ' (' . $MATRIX[$team][$prob]['time'] . ' + ' .
-				               $MATRIX[$team][$prob]['penalty'] . ')';
+			if( $matrix[$team][$prob]['is_correct'] ) {
+				$str .= ' (' . $matrix[$team][$prob]['time'] . ' + ' .
+				               $matrix[$team][$prob]['penalty'] . ')';
 			}
 			echo '>' . jurylink('team.php?id=' . urlencode($team) .
 								'&amp;restrict=probid:' . urlencode($prob),
@@ -389,40 +424,36 @@ function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
 		echo "</tr>\n";
 	}
 	echo "</tbody>\n\n<tbody>\n";
-	
+
 	// print a summaryline
 	echo '<tr id="scoresummary" title="#submitted / #correct / fastest time">' .
 		'<td title="total teams">' .
 		jurylink(null,count($teams)) . '</td>' .
 		( $SHOW_AFFILIATIONS ? '<td class="scoreaffil" title="#affiliations / #countries">' .
-		  jurylink('team_affiliations.php',count($SUMMARY['affils']) . ' / ' .
-				   count($SUMMARY['countries'])) . '</td>' : '' ) .
+		  jurylink('team_affiliations.php',count($summary['affils']) . ' / ' .
+				   count($summary['countries'])) . '</td>' : '' ) .
 		'<td title=" ">' . jurylink(null,'Summary') . '</td>' .
-		'<td title="total solved" class="scorenc">' . jurylink(null,$SUMMARY['num_correct'])  . '</td><td title=" "></td>';
+		'<td title="total solved" class="scorenc">' . jurylink(null,$summary['num_correct'])  . '</td><td title=" "></td>';
 
 	foreach( array_keys($probs) as $prob ) {
-		$str = $SUMMARY['problems'][$prob]['num_submissions'] . ' / ' .
-		       $SUMMARY['problems'][$prob]['num_correct'] . ' / ' .
-			   ( isset($SUMMARY['problems'][$prob]['best_time']) ? 
-				 $SUMMARY['problems'][$prob]['best_time'] : '-' );
+		$str = $summary['problems'][$prob]['num_submissions'] . ' / ' .
+		       $summary['problems'][$prob]['num_correct'] . ' / ' .
+			   ( isset($summary['problems'][$prob]['best_time']) ?
+				 $summary['problems'][$prob]['best_time'] : '-' );
 		echo '<td>' .
 			jurylink('problem.php?id=' . urlencode($prob),$str) .
 			'</td>';
 	}
 	echo "</tr>\n</tbody>\n</table>\n\n";
 
-	$categs = $DB->q('SELECT * FROM team_category ' .
-	                 (IS_JURY ? '' : 'WHERE visible = 1 ' ) .
-	                 'ORDER BY sortorder,name,categoryid');
-
 	// only print legend when there's more than one category
-	if ( $categs->count() > 1 ) {
+	if ( count($categs) > 1 ) {
 		echo "<p><br /><br /></p>\n<table id=\"legend\" class=\"scoreboard" .
 			(IS_JURY ? ' scoreboard_jury' : '') . "\">\n" .
 			"<thead><tr><th scope=\"col\">" .
 			jurylink('team_categories.php','Legend') .
 			"</th></tr></thead>\n<tbody>\n";
-		while ( $cat = $categs->next() ) {
+		foreach( $categs as $cat ) {
 			echo '<tr' . (!empty($cat['color']) ? ' style="background: ' .
 				          $cat['color'] . ';"' : '') . '>' .
 				'<td align="center" class="scoretn">' .
@@ -445,6 +476,17 @@ function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
 	return;
 }
 
+/**
+ * Very simple wrapper around the scoreboard data-generating and
+ * rendering functions.
+ */
+function putScoreBoard($cdata, $myteamid = null, $static = FALSE) {
+
+	$sdata = genScoreBoard($cdata);
+
+	renderScoreBoard($cdata,$sdata,$myteamid,$static);
+}
+
 /**
  * Output a team row from the scoreboard based on the cached data in
  * table 'scoreboard'.
@@ -455,7 +497,7 @@ function putTeamRow($cdata, $teamid) {
 
 	if ( empty( $cdata )  || difftime($cdata['starttime'],now()) > 0 ) return;
 	$cid = $cdata['cid'];
-	
+
 	echo '<table class="scoreboard">' . "\n";
 
 	$probs = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY, probid, name, color
@@ -477,13 +519,15 @@ function putTeamRow($cdata, $teamid) {
 	while ( $srow = $scoredata->next() ) {
 		// skip this row if the problem is not known by us
 		if ( ! array_key_exists ( $srow['probid'], $probs ) ) continue;
-	
+
+		$penalty = calcPenaltyTime( $srow['is_correct'], $srow['submissions'] );
+
 		// fill our matrix with the scores from the database,
 		$MATRIX[$srow['probid']] = array (
 			'is_correct'      => (bool) $srow['is_correct'],
 			'num_submissions' => $srow['submissions'],
 			'time'            => $srow['totaltime'],
-			'penalty'         => $srow['penalty'] );
+			'penalty'         => $penalty );
 	}
 
 	$SUMMARY = array('num_correct' => 0, 'total_time' => 0);
@@ -499,12 +543,13 @@ function putTeamRow($cdata, $teamid) {
 		}
 		echo '<tr><td class="probid" title="' .
 			htmlspecialchars($probdata['name']) . '">' .
-			(!empty($probdata['color']) ? '<span style="color: ' .
-			       htmlspecialchars($probdata['color']) . ';">' .
-			       BALLOON_SYM . '</span> ' : '' ) .
+			( !empty($probdata['color']) ?
+			  '<img style="background-color: ' . htmlspecialchars($probdata['color']) .
+			  ';" alt="problem colour ' . htmlspecialchars($probdata['color']) .
+			  '" src="../images/circle.png" /> ' : '' ) .
 			htmlspecialchars($prob) . '</td><td class="';
 		// CSS class for correct/incorrect/neutral results
-		if( $pdata['is_correct'] ) { 
+		if( $pdata['is_correct'] ) {
 			echo 'score_correct';
 		} elseif ( $pdata['num_submissions'] > 0 ) {
 			echo 'score_incorrect';
@@ -541,7 +586,7 @@ function jurylink($target, $content) {
 	}
 	$res .= $content;
 	if ( IS_JURY ) $res .= '</a>';
-	
+
 	return $res;
 }
 
