@@ -3,7 +3,7 @@
    Copyright (C) 2008 Jan Kuipers
    Copyright (C) 2009 Jaap Eldering (eldering@a-eskwadraat.nl).
 
-   $Id: checktestdata.cpp 3309 2010-08-09 19:06:26Z eldering $
+   $Id: checktestdata.cpp 3304 2010-08-09 18:49:11Z eldering $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,8 +27,15 @@
    integer  := 0|-?[1-9][0-9]*
    variable := [a-z][a-z0-9]*
    value    := <integer> | <variable>
+   compare  := '<' | '>' | '<=' | '>=' | '==' | '!='
    expr     := <term> | <expr> [+-] <term>
    term     := <value> | '-' <term> | '(' <expr> ')' | <term> [*%/] <term>
+   test     := '!' <test> | '(' <test> ')' | <test> [&|] <test> |
+               <expr> <compare> <expr> | 'MATCH(' <string> str ')' | 'ISEOF'
+
+      MATCH and ISEOF are special keyword that return whether the next
+      character matches any of 'str', respectively if end-of-file has
+      been reached.
 
    string   := ".*"
 
@@ -52,7 +59,7 @@
 
    EOF
 
-      Matches end-of-file. This implicitly added at the end of each
+      Matches end-of-file. This is implicitly added at the end of each
       program and must match exactly: no extra data may be present.
 
    INT(<expr> min, <expr> max [, <variable> name])
@@ -75,19 +82,35 @@
       times and optionally match 'separator' command (count-1) times
       in between. The value of count must fit in a unsigned 32 bit int.
 
+   WHILE(<test> condition [,<command> separator]) [<command>...] END
+
+      Repeat the commands as long as 'condition' is true. Optionally
+      match 'separator' command between two consecutive iterations.
+
  */
 
+#include "../etc/config.h"
+
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <vector>
 #include <string>
 #include <map>
+#include <set>
 #include <ctype.h>
 #include <getopt.h>
 #include <stdarg.h>
+#ifdef HAVE_BOOST_REGEX
 #include <boost/regex.hpp>
+#else
+#error "Libboost regex library not available."
+#endif
+#ifdef HAVE_GMPXX_H
 #include <gmpxx.h>
+#else
+#error "LibGMP C++ extensions not available."
+#endif
 
 #include "parser.h"
 
@@ -95,28 +118,37 @@ using namespace std;
 
 #define PROGRAM "checktestdata"
 #define AUTHORS "Jan Kuipers, Jaap Eldering"
-#define VERSION "$Rev: 3309 $"
+#define VERSION "$Rev: 3304 $"
+
+const int exit_testdata = 1;
+const int exit_failure  = 2;
 
 const int display_before_error = 65;
 const int display_after_error  = 10;
 
-size_t prognr, datanr, linenr, charnr;
+size_t prognr, datanr, linenr, charnr, extra_ws;
 command currcmd;
 
 string data;
 vector<command> program;
 map<string,mpz_class> variable;
+set<string> loop_cmds;
+// List of loop starting commands like REP, initialized in main().
 
 char *progname;
 char *progfile;
 char *datafile;
 
+int whitespace_ok;
 int debugging;
+int quiet;
 int show_help;
 int show_version;
 
 struct option const long_opts[] = {
+	{"whitespace-ok", no_argument, NULL,         'w'},
 	{"debug",   no_argument,       NULL,         'd'},
+	{"quiet",   no_argument,       NULL,         'q'},
 	{"help",    no_argument,       &show_help,    1 },
 	{"version", no_argument,       &show_version, 1 },
 	{ NULL,     0,                 NULL,          0 }
@@ -139,9 +171,13 @@ void usage()
 "Check TESTDATA according to specification in PROGRAM file.\n"
 "If TESTDATA is '-' or not specified, read from stdin.\n"
 "\n"
-"  -d, --debug        enable extra debugging output\n"
-"      --help         display this help and exit\n"
-"      --version      output version information and exit\n"
+"  -w, --whitespace-ok  whitespace changes are accepted, including heading\n"
+"                         and trailing whitespace, but not newlines;\n"
+"                         be careful: extra whitespace matches greedily!\n"
+"  -d, --debug          enable extra debugging output\n"
+"  -q, --quiet          don't display testdata error messages: test exitcode\n"
+"      --help           display this help and exit\n"
+"      --version        output version information and exit\n"
 "\n",progname);
 }
 
@@ -172,7 +208,7 @@ void readprogram(istream &in)
 	Parser parseprog(in);
 	if ( parseprog.parse()!=0 ) {
 		cerr << "parse error reading " << progfile << endl;
-		exit(1);
+		exit(exit_failure);
 	}
 
 	// Add (implicit) EOF command at end of input
@@ -181,16 +217,17 @@ void readprogram(istream &in)
 	// Check for correct REP ... END nesting
 	int replevel = 0;
 	for (size_t i=0; i<program.size(); i++) {
-		if ( program[i].name()=="REP" ) replevel++;
+		if ( program[i].name()=="WHILE" ||
+		     program[i].name()=="REP" ) replevel++;
 		if ( program[i].name()=="END" ) replevel--;
 		if ( replevel<0 ) {
-			cerr << "unbalanced REP/END statements" << endl;
-			exit(1);
+			cerr << "unbalanced WHILE/REP/END statements" << endl;
+			exit(exit_failure);
 		}
 	}
 	if ( replevel!=0 ) {
-		cerr << "unbalanced REP/END statements" << endl;
-		exit(1);
+		cerr << "unbalanced WHILE/REP/END statements" << endl;
+		exit(exit_failure);
 	}
 }
 
@@ -202,7 +239,7 @@ void readtestdata(istream &in)
 	ss << in.rdbuf();
 	if ( in.fail() ) {
 		cerr << "error reading " << datafile << endl;
-		exit(1);
+		exit(exit_failure);
 	}
 
  	data = ss.str();
@@ -215,13 +252,15 @@ void error()
 
 	debug("error at datanr = %d, %d - %d\n",(int)datanr,(int)fr,(int)to);
 
-	cout << data.substr(fr,to-fr) << endl;
-	cout << string(min(charnr,(size_t)display_before_error),' ') << "^" << endl << endl;
+	if ( !quiet ) {
+		cout << data.substr(fr,to-fr) << endl;
+		cout << string(min(charnr,(size_t)display_before_error),' ') << "^" << endl << endl;
 
-	cout << "ERROR: line " << linenr << " character " << charnr;
-	cout << " of testdata doesn't match " << currcmd << endl << endl;
+		cout << "ERROR: line " << linenr << " character " << charnr;
+		cout << " of testdata doesn't match " << currcmd << endl << endl;
+	}
 
-	exit(1);
+	exit(exit_testdata);
 }
 
 mpz_class value(string x)
@@ -230,7 +269,7 @@ mpz_class value(string x)
 	if ( isalpha(x[0]) ) {
 		if ( variable.count(x) ) return variable[x];
 		cerr << "variable " << x << " undefined in " << program[prognr] << endl;
-		exit(1);
+		exit(exit_failure);
 	}
 
 	return mpz_class(x);
@@ -240,32 +279,103 @@ mpz_class eval(expr e)
 {
 	debug("eval op='%c', val='%s', #args=%d",e.op,e.val.c_str(),(int)e.args.size());
 	switch ( e.op ) {
+	case ' ': return value(e.val);
 	case 'n': return -eval(e.args[0]);
-	case '(': return eval(e.args[0]);
 	case '+': return eval(e.args[0]) + eval(e.args[1]);
 	case '-': return eval(e.args[0]) - eval(e.args[1]);
 	case '*': return eval(e.args[0]) * eval(e.args[1]);
 	case '/': return eval(e.args[0]) / eval(e.args[1]);
 	case '%': return eval(e.args[0]) % eval(e.args[1]);
-	default:  return value(e.val);
+	default:
+		cerr << "unknown arithmetic operator " << e.op << " in "
+		     << program[prognr] << endl;
+		exit(exit_failure);
 	}
 }
 
+bool compare(args_t cmp)
+{
+	string op = cmp[0].val;
+	mpz_class l = eval(cmp[1]);
+	mpz_class r = eval(cmp[2]);
+
+	if ( op=="<"  ) return l<r;
+	if ( op==">"  ) return l>r;
+	if ( op=="<=" ) return l<=r;
+	if ( op==">=" ) return l>=r;
+	if ( op=="==" ) return l==r;
+	if ( op=="!=" ) return l!=r;
+
+	cerr << "unknown compare operator " << op << " in "
+	     << program[prognr] << endl;
+	exit(exit_failure);
+}
+
+bool dotest(test t)
+{
+	debug("test op='%c', #args=%d",t.op,(int)t.args.size());
+	switch ( t.op ) {
+	case '!': return !dotest(t.args[0]);
+	case '&': return dotest(t.args[0]) && dotest(t.args[1]);
+	case '|': return dotest(t.args[0]) || dotest(t.args[1]);
+	case 'E': return datanr>=data.size();
+	case 'M': return datanr<data.size() && t.args[0].val.find(data[datanr])!=string::npos;
+	case '?': return compare(t.args);
+	default:
+		cerr << "unknown test " << t.op << " in " << program[prognr] << endl;
+		exit(exit_failure);
+	}
+}
+
+int isspace_notnewline(char c) { return isspace(c) && c!='\n'; }
+
+void readwhitespace()
+{
+	while ( datanr<data.size() && isspace_notnewline(data[datanr]) ) {
+		datanr++;
+		charnr++;
+		extra_ws++;
+	}
+}
+
+void checkspace()
+{
+	if ( datanr>=data.size() ) error();
+
+	if ( whitespace_ok ) {
+		// First check at least one space-like character
+		if ( !isspace_notnewline(data[datanr++]) ) error();
+		charnr++;
+		// Then greedily read non-newline whitespace
+		readwhitespace();
+	} else {
+		if ( data[datanr++]!=' ' ) error();
+		charnr++;
+	}
+}
+
+void checknewline()
+{
+	// Trailing whitespace before newline
+	if ( whitespace_ok ) readwhitespace();
+
+	if ( datanr>=data.size() || data[datanr++]!='\n' ) error();
+	linenr++;
+	charnr=0;
+
+	// Leading whitespace after newline
+	if ( whitespace_ok ) readwhitespace();
+
+}
+
 void checktoken(command cmd)
 {
 	currcmd = cmd;
 	debug("checking token %s at %d,%d",cmd.name().c_str(),linenr,charnr);
 
-	if ( cmd.name()=="SPACE" ) {
-		if ( datanr>=data.size() || data[datanr++]!=' ' ) error();
-		charnr++;
-	}
+	if ( cmd.name()=="SPACE" ) checkspace();
 
-	else if ( cmd.name()=="NEWLINE" ) {
-		if ( datanr>=data.size() || data[datanr++]!='\n' ) error();
-		linenr++;
-		charnr=0;
-	}
+	else if ( cmd.name()=="NEWLINE" ) checknewline();
 
 	else if ( cmd.name()=="INT" ) {
 		// Accepts format (0|-?[1-9][0-9]*), i.e. no leading zero's
@@ -333,7 +443,8 @@ void checktoken(command cmd)
 	}
 
 	else {
-		error();
+		cerr << "unknown command " << program[prognr] << endl;
+		exit(exit_failure);
 	}
 }
 
@@ -347,34 +458,49 @@ void checktestdata()
 			return;
 		}
 
-		else if ( cmd.name()=="REP" ) {
-			mpz_class n = eval(cmd.args[0]);
-			if ( !n.fits_ulong_p() ) {
-				cerr << "'" << n << "' does not fit in unsigned long in "
-				     << program[prognr] << endl;
-				exit(1);
-			}
-			long long times = n.get_ui();
+		else if ( cmd.name()=="REP" ||
+		          cmd.name()=="WHILE" ) {
 
-			if ( times==0 ) {
-				int replevel = 0;
-				do {
-					command cmd = program[prognr++];
-					if ( cmd.name()=="REP" ) replevel++;
-					if ( cmd.name()=="END" ) replevel--;
-				}
-				while ( replevel>0 );
-			}
-			else {
-				int loopstart = prognr+1;
+			// Current and maximum loop iterations.
+			unsigned long long i = 0, times = ULLONG_MAX;
 
-				while ( times-- ) {
-					debug("REP times = %lld",times+1);
-					prognr = loopstart;
-					checktestdata();
-					if ( times && cmd.nargs()>=2 ) checktoken(cmd.args[1]);
+			if ( cmd.name()=="REP" ) {
+				mpz_class n = eval(cmd.args[0]);
+				if ( !n.fits_ulong_p() ) {
+					cerr << "'" << n << "' does not fit in an unsigned long in "
+						 << program[prognr] << endl;
+					exit(exit_failure);
 				}
+				times = n.get_ui();
 			}
+
+			// Begin and end of loop commands
+			int loopbegin, loopend;
+
+			loopbegin = loopend = prognr + 1;
+
+			for(int looplevel=1; looplevel>0; ++loopend) {
+				string cmdstr = program[loopend].name();
+				if ( loop_cmds.count(cmdstr) ) looplevel++;
+				if ( cmdstr=="END" ) looplevel--;
+			}
+
+			// Run loop...
+			debug("running %s loop, commands %d - %d, max. times = %lld",
+			      cmd.name().c_str(),loopbegin,loopend,times);
+
+			while ( (cmd.name()=="REP"   && i<times) ||
+			        (cmd.name()=="WHILE" && dotest(cmd.args[0])) ) {
+
+				debug("loop iteration %lld/%lld",i+1,times);
+				prognr = loopbegin;
+				if ( i>0 && cmd.nargs()>=2 ) checktoken(cmd.args[1]);
+				checktestdata();
+				i++;
+			}
+
+			// And skip to end of loop
+			prognr = loopend;
 		}
 
 		else if ( cmd.name()=="END" ) {
@@ -396,22 +522,29 @@ int main(int argc, char **argv)
 	progname = argv[0];
 
 	/* Parse command-line options */
-	debugging = show_help = show_version = 0;
+	whitespace_ok = 0;
+	debugging = quiet = show_help = show_version = 0;
 	opterr = 0;
-	while ( (opt = getopt_long(argc,argv,"+d",long_opts,(int *) 0))!=-1 ) {
+	while ( (opt = getopt_long(argc,argv,"+wdq",long_opts,(int *) 0))!=-1 ) {
 		switch ( opt ) {
 		case 0:   /* long-only option */
 			break;
+		case 'w':
+			whitespace_ok = 1;
+			break;
 		case 'd':
 			debugging = 1;
 			break;
+		case 'q':
+			quiet = 1;
+			break;
 		case ':': /* getopt error */
 		case '?':
 			printf("unknown option or missing argument `%c'",optopt);
-			return 1;
+			exit(exit_failure);
 		default:
 			printf("getopt returned character code `%c' ??",(char)opt);
-			return 1;
+			exit(exit_failure);
 		}
 	}
 
@@ -423,13 +556,13 @@ int main(int argc, char **argv)
 	if ( argc<=optind ) {
 		printf("Error: no PROGRAM file specified.\n");
 		usage();
-		return 1;
+		return exit_failure;
 	}
 	progfile = argv[optind];
 	ifstream prog(progfile);
 	if ( prog.fail() ) {
 		cerr << "error opening " << progfile << endl;
-		exit(1);
+		exit(exit_failure);
 	}
 	readprogram(prog);
 	prog.close();
@@ -447,7 +580,7 @@ int main(int argc, char **argv)
 		ifstream fin(datafile);
 		if ( fin.fail() ) {
 			cerr << "error opening " << datafile << endl;
-			exit(1);
+			exit(exit_failure);
 		}
 		readtestdata(fin);
 		fin.close();
@@ -455,10 +588,20 @@ int main(int argc, char **argv)
 
 	linenr = charnr = 0;
 	datanr = prognr = 0;
+	extra_ws = 0;
+
+	// Initialize loop_cmds here, as a set cannot be initialized on
+	// declaration.
+	loop_cmds.insert("REP");
+	loop_cmds.insert("WHILE");
+
+	// If we ignore whitespace, skip leading whitespace on first line
+	// as a special case; other lines are handled by checknewline().
+	if ( whitespace_ok ) readwhitespace();
 
 	checktestdata();
 
-	cout << "testdata ok!" << endl;
+	if ( !quiet ) cout << "testdata ok!" << endl;
 
 	return 0;
 }
