@@ -2,7 +2,7 @@
 /**
  * View of one contest.
  *
- * $Id: contest.php 3169 2010-05-16 15:52:48Z eldering $
+ * $Id: contest.php 3209 2010-06-12 00:13:43Z eldering $
  *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
@@ -51,10 +51,13 @@ if ( IS_ADMIN && !empty($_GET['cmd']) ):
 <tr><td><label for="data_0__endtime_">End time:</label></td>
 <td><?php echo addInput('data[0][endtime]', @$row['endtime'], 20, 19)?> (yyyy-mm-dd hh:mm:ss)</td></tr>
 
-
 <tr><td><label for="data_0__unfreezetime_">Scoreboard unfreeze time:</label></td>
 <td><?php echo addInput('data[0][unfreezetime]', @$row['unfreezetime'], 20, 19)?> (yyyy-mm-dd hh:mm:ss)</td></tr>
 
+<tr><td>Enabled:</td><td>
+<?php echo addRadioButton('data[0][enabled]', (!isset($row['enabled']) ||  $row['enabled']), 1)?> <label for="data_0__enabled_1">yes</label>
+<?php echo addRadioButton('data[0][enabled]', ( isset($row['enabled']) && !$row['enabled']), 0)?> <label for="data_0__enabled_0">no</label></td></tr>
+
 </table>
 
 <?php
@@ -80,6 +83,9 @@ echo "<h1>Contest: ".htmlspecialchars($data['contestname'])."</h1>\n\n";
 if ( $cid == $data['cid'] ) {
 	echo "<p><em>This is the current contest.</em></p>\n\n";
 }
+if ( !$data['enabled'] ) {
+	echo "<p><em>This contest is disabled.</em></p>\n\n";
+}
 
 echo "<table>\n";
 echo '<tr><td scope="row">CID:</td><td>c' .
@@ -93,7 +99,7 @@ echo '<tr><td scope="row">Activate time:</td><td>' .
 echo '<tr><td scope="row">Start time:</td><td>' .
 	htmlspecialchars($data['starttime']) .
 	"</td></tr>\n";
-echo '<tr><td scope="row">Scoreboard freeze:</td><td>' . 
+echo '<tr><td scope="row">Scoreboard freeze:</td><td>' .
 	(empty($data['freezetime']) ? "-" : htmlspecialchars(@$data['freezetime'])) .
 	"</td></tr>\n";
 echo '<tr><td scope="row">End time:</td><td>' .
@@ -105,7 +111,7 @@ echo '<tr><td scope="row">Scoreboard unfreeze:</td><td>' .
 echo "</table>\n\n";
 
 if ( IS_ADMIN ) {
-	echo "<p>" . 
+	echo "<p>" .
 		editLink('contest',$data['cid']) . "\n" .
 		delLink('contest','cid',$data['cid']) ."</p>\n\n";
 }
