@@ -3,14 +3,14 @@
 /**
  * Scoreboard
  *
- * $Id: scoreboard.php 2550 2008-12-01 23:07:13Z eldering $
+ * $Id: scoreboard.php 3118 2010-02-22 20:23:55Z kink $
  *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
 
 require('init.php');
-$refresh = '30;url=' . getBaseURI() . 'jury/scoreboard.php';
+$refresh = '30;url=scoreboard.php';
 $title = 'Scoreboard';
 $printercss = TRUE;
 
