@@ -2,7 +2,7 @@
 /**
  * View team details
  *
- * $Id: team.php 3169 2010-05-16 15:52:48Z eldering $
+ * $Id: team.php 3209 2010-06-12 00:13:43Z eldering $
  *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
@@ -10,24 +10,25 @@
 
 $pagename = basename($_SERVER['PHP_SELF']);
 
-$id = @$_REQUEST['id'];
-
 require('init.php');
 
+$id = @$_REQUEST['id'];
+$title = 'Team '.htmlspecialchars(@$id);
+
+if ( ! preg_match('/^\w*$/', $id) ) error("Invalid team id");
+
 if ( isset($_GET['cmd'] ) ) {
 	$cmd = $_GET['cmd'];
 } else {
-	$refresh = '15;url='.getBaseURI().'jury/'.$pagename.'?id='.urlencode($id).
+	$refresh = '15;url='.$pagename.'?id='.urlencode($id).
 		(isset($_GET['restrict'])?'&restrict='.urlencode($_GET['restrict']):'');
 }
 
-$title = 'Team '.htmlspecialchars(@$id);
-
 require(LIBWWWDIR . '/header.php');
 require(LIBWWWDIR . '/forms.php');
 
 if ( IS_ADMIN && !empty($cmd) ):
-	
+
 	echo "<h2>" . htmlspecialchars(ucfirst($cmd)) . " team</h2>\n\n";
 
 	echo addForm('edit.php');
@@ -64,8 +65,8 @@ $amap[''] = 'none';
 echo addSelect('data[0][affilid]', $amap, @$row['affilid'], true);
 ?>
 </td></tr>
-<tr><td><label for="data_0__ipaddress_">IP address:</label></td>
-<td><?php echo addInput('data[0][ipaddress]', @$row['ipaddress'], 35, 32)?> (or hostname)</td></tr>
+<tr><td><label for="data_0__authtoken_">Auth token:</label></td>
+<td><?php echo addInput('data[0][authtoken]', @$row['authtoken'], 35, 255)?></td></tr>
 <tr><td><label for="data_0__room_">Room:</label></td>
 <td><?php echo addInput('data[0][room]', @$row['room'], 10, 15)?></td></tr>
 <tr><td valign="top"><label for="data_0__comments_">Comments:</label></td>
@@ -116,14 +117,8 @@ if ( is_readable($teamimage) ) {
 <div class="col1"><table>
 <tr><td scope="row">Login:     </td><td class="teamid"><?php echo $row['login']?></td></tr>
 <tr><td scope="row">Name:      </td><td><?php echo htmlspecialchars($row['name'])?></td></tr>
-<tr><td scope="row">Has passwd:</td><td><?php echo (isset($row['passwd']) ? 'yes':'no')?>
-<?php if ( IS_ADMIN ): ?>
-	<a href="genpasswds.php?forteam=<?php echo $id?>"><img src="../images/edit.png"
-	alt="edit" title="set password" class="picto" /></a>
-<?php endif; ?>
-	</td></tr>
-<tr><td scope="row">Host:</td><td><?php echo @$row['ipaddress'] ? htmlspecialchars($row['ipaddress']).
-	' - '.(@$row['hostname'] ? printhost($row['hostname'], TRUE):''):'-'?></td></tr>
+<tr><td scope="row">Host:</td><td><?php echo
+	(@$row['hostname'] ? printhost($row['hostname'], TRUE):'') ?></td></tr>
 <?php if (!empty($row['room'])): ?>
 <tr><td scope="row">Room:</td><td><?php echo htmlspecialchars($row['room'])?></td></tr>
 <?php endif; ?>
