@@ -2,7 +2,7 @@
 /**
  * View a row in team_affiliation: an institution, company etc
  *
- * $Id: team_affiliation.php 3169 2010-05-16 15:52:48Z eldering $
+ * $Id: team_affiliation.php 3209 2010-06-12 00:13:43Z eldering $
  *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
@@ -10,10 +10,13 @@
 
 $pagename = basename($_SERVER['PHP_SELF']);
 
-$id = @$_GET['id'];
-
 require('init.php');
 
+$id = @$_GET['id'];
+$title = "Affiliation: " .htmlspecialchars(@$id);
+
+if ( ! preg_match('/^\w*$/', $id) ) error("Invalid affiliation id");
+
 $cmd = @$_GET['cmd'];
 
 if ( IS_ADMIN && ($cmd == 'add' || $cmd == 'edit') ) {
@@ -70,16 +73,12 @@ echo addHidden('cmd', $cmd) .
 }
 
 
-
-
-if ( ! $id ) error("Missing or invalid affiliation id");
-$title = "Affiliation: " .htmlspecialchars(@$id);
-
-
 require(LIBWWWDIR . '/header.php');
 
 $data = $DB->q('TUPLE SELECT * FROM team_affiliation WHERE affilid = %s', $id);
 
+if ( ! $data ) error("Missing or invalid affiliation id");
+
 $affillogo = "../images/affiliations/" . urlencode($data['affilid']) . ".png";
 $countryflag = "../images/countries/" . urlencode($data['country']) . ".png";
 
@@ -114,7 +113,7 @@ if ( !empty($data['comments']) ) {
 echo "</table>\n\n";
 
 if ( IS_ADMIN ) {
-	echo "<p>" . 
+	echo "<p>" .
 		editLink('team_affiliation', $data['affilid']) . "\n" .
 		delLink('team_affiliation', 'affilid', $data['affilid']) . "</p>\n\n";
 }
@@ -123,16 +122,16 @@ echo "<h2>Teams from " . htmlspecialchars($data['name']) . "</h2>\n\n";
 
 $teams = $DB->q('SELECT login,name FROM team WHERE affilid = %s', $id);
 if ( $teams->count() == 0 ) {
-	echo "<p><em>no teams</em></p>\n\n";
+	echo "<p class=\"nodata\">no teams</p>\n\n";
 } else {
-	echo "<table>\n<thead>\n" .
+	echo "<table class=\"list\">\n<thead>\n" .
 		"<tr><th scope=\"col\">login</th><th scope=\"col\">teamname</th></tr>\n" .
 		"</thead>\n<tbody>\n";
 	while ($team = $teams->next()) {
-		echo "<tr><td class=\"teamid\"><a href=\"team.php?id=" .
-			urlencode($team['login']) . "\">" .
-			htmlspecialchars($team['login']) . "</a></td><td>" .
-			htmlspecialchars($team['name']) . "</td></tr>\n";
+		$link = '<a href="team.php?id=' . urlencode($team['login']) . '">';
+		echo "<tr><td class=\"teamid\">" .
+		$link . htmlspecialchars($team['login']) . "</a></td><td>" .
+		$link . htmlspecialchars($team['name']) . "</a></td></tr>\n";
 	}
 	echo "</tbody>\n</table>\n\n";
 }
