@@ -2,7 +2,7 @@
 /**
  * View the teams
  *
- * $Id: teams.php 3367 2010-08-19 19:53:48Z eldering $
+ * $Id: teams.php 3360 2010-08-19 19:26:55Z eldering $
  *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
@@ -32,7 +32,7 @@ require(LIBWWWDIR . '/header.php');
 echo "<h1>Teams</h1>\n\n";
 
 if( $teams->count() == 0 ) {
-	echo "<p><em>No teams defined</em></p>\n\n";
+	echo "<p class=\"nodata\">No teams defined</p>\n\n";
 } else {
 	echo "<table class=\"list sortable\">\n<thead>\n" .
 		"<tr><th scope=\"col\">login</th><th scope=\"col\">teamname</th>" .
@@ -56,31 +56,26 @@ if( $teams->count() == 0 ) {
 			$status = 3;
 			$numcor = (int)$ncorrect[$row['login']]['cnt'];
 		}
-		
+		$link = '<a href="team.php?id='.urlencode($row['login']) . '">';
 		echo "<tr class=\"category" . (int)$row['categoryid'] . "\">".
-			"<td class=\"teamid\"><a href=\"team.php?id=".urlencode($row['login'])."\">".
+			"<td class=\"teamid\">" . $link .
 				htmlspecialchars($row['login'])."</a></td>".
-			"<td><a href=\"team.php?id=".htmlspecialchars($row['login'])."\">".
+			"<td>" . $link .
 				htmlspecialchars($row['name'])."</a></td>".
-			"<td><a href=\"team_category.php?id=".urlencode($row['categoryid'])."\">".
+			"<td>" . $link .
 				htmlspecialchars($row['catname'])."</a></td>".
-			"<td title=\"".htmlspecialchars($row['affname'])."\">".
-				"<a href=\"team_affiliation.php?id=".urlencode($row['affilid'])."\">".
-				htmlspecialchars($row['affilid'])."</a></td><td title=\"";
-		
-		if ( @$row['ipaddress'] ) {
-			$host = (empty($row['hostname'])?'':$row['hostname']);
-			echo htmlspecialchars($row['ipaddress']);
-			if ( empty($host) ) {
-				echo "\">" . htmlspecialchars($row['ipaddress']);
-			} else {
-				echo " - " . htmlspecialchars($host) . "\">" .
-					printhost($host);
-			}
+			"<td title=\"".htmlspecialchars($row['affname'])."\">" . $link .
+				($row['affilid'] ? htmlspecialchars($row['affilid']) : '&nbsp;') .
+			"</a></td><td title=\"";
+
+		if ( @$row['hostname'] ) {
+			echo htmlspecialchars($row['hostname']) . "\">" . $link .
+			    printhost($row['hostname']);
 		} else {
-			echo "\">-";
+			echo "\">" . $link . "-";
 		}
-		echo "</td><td>".htmlspecialchars($row['room'])."</td>";
+		echo "</a></td><td>" . $link .
+			($row['room'] ? htmlspecialchars($row['room']) : '&nbsp;') . "</a></td>";
 		echo "<td class=\"";
 		switch ( $status ) {
 		case 0: echo 'team-nocon" title="no connections made"';
@@ -92,10 +87,11 @@ if( $teams->count() == 0 ) {
 		case 3: echo 'team-ok" title="correct submission(s)"';
 			break;
 		}
-		echo ">".BALLOON_SYM."</td>";
+		// TODO? want to link this symbol aswell? need to do some css magic to retain color
+		echo ">" . CIRCLE_SYM . "</td>";
 		echo "<td align=\"right\" title=\"$numcor correct / $numsub submitted\">$numcor / $numsub</td>";
 		if ( IS_ADMIN ) {
-			echo "<td>" .
+			echo "<td class=\"editdel\">" .
 				editLink('team', $row['login']) . " " .
 				delLink('team','login',$row['login']) . "</td>";
 		}
